@@ -8,10 +8,12 @@ from twisted.internet.protocol import Protocol
 
 from twisted.python.failure import Failure
 
+from twisted.trial.unittest import TestCase
+
 from twisted.web.client import Agent, ResponseFailed
 from twisted.web.http_headers import Headers
 
-from treq.test.util import TestCase, with_clock
+from treq.test.util import with_clock
 
 from treq.client import (
     HTTPClient, _BodyBufferingProtocol, _BufferedResponse
@@ -35,10 +37,28 @@ class HTTPClientTests(TestCase):
         body = self.FileBodyProducer.mock_calls[0][1][0]
         self.assertEqual(body.read(), expected)
 
-    def test_request_uri_idn(self):
-        self.client.request('GET', u'http://‽.net')
+    def test_post(self):
+        self.client.post('http://example.com/')
         self.agent.request.assert_called_once_with(
-            b'GET', b'http://xn--fwg.net',
+            b'POST', b'http://example.com/',
+            Headers({b'accept-encoding': [b'gzip']}), None)
+
+    def test_request_uri_idn(self):
+        self.client.request('GET', u'http://č.net')
+        self.agent.request.assert_called_once_with(
+            b'GET', b'http://xn--bea.net',
+            Headers({b'accept-encoding': [b'gzip']}), None)
+
+    def test_request_uri_idn_params(self):
+        """
+        A URL that contains non-ASCII characters can be augmented with
+        querystring parameters.
+
+        This reproduces treq #264.
+        """
+        self.client.request('GET', u'http://č.net', params={'foo': 'bar'})
+        self.agent.request.assert_called_once_with(
+            b'GET', b'http://xn--bea.net/?foo=bar',
             Headers({b'accept-encoding': [b'gzip']}), None)
 
     def test_request_case_insensitive_methods(self):
@@ -63,6 +83,89 @@ class HTTPClientTests(TestCase):
             b'GET', b'http://example.com/?foo=bar',
             Headers({b'accept-encoding': [b'gzip']}), None)
 
+    def test_request_tuple_query_value_coercion(self):
+        """
+        treq coerces non-string values passed to *params* like
+        `urllib.urlencode()`
+        """
+        self.client.request('GET', 'http://example.com/', params=[
+            ('text', u'A\u03a9'),
+            ('text-seq', [u'A\u03a9']),
+            ('bytes', [b'ascii']),
+            ('bytes-seq', [b'ascii']),
+            ('native', ['native']),
+            ('native-seq', ['aa', 'bb']),
+            ('int', 1),
+            ('int-seq', (1, 2, 3)),
+            ('none', None),
+            ('none-seq', [None, None]),
+        ])
+
+        self.agent.request.assert_called_once_with(
+            b'GET',
+            (
+                b'http://example.com/?'
+                b'text=A%CE%A9&text-seq=A%CE%A9'
+                b'&bytes=ascii&bytes-seq=ascii'
+                b'&native=native&native-seq=aa&native-seq=bb'
+                b'&int=1&int-seq=1&int-seq=2&int-seq=3'
+                b'&none=None&none-seq=None&none-seq=None'
+            ),
+            Headers({b'accept-encoding': [b'gzip']}),
+            None,
+        )
+
+    def test_request_tuple_query_param_coercion(self):
+        """
+        treq coerces non-string param names passed to *params* like
+        `urllib.urlencode()`
+        """
+        self.client.request('GET', 'http://example.com/', params=[
+            (u'text', u'A\u03a9'),
+            (b'bytes', ['ascii']),
+            ('native', 'native'),
+            (1, 'int'),
+            (None, ['none']),
+        ])
+
+        self.agent.request.assert_called_once_with(
+            b'GET',
+            (
+                b'http://example.com/'
+                b'?text=A%CE%A9&bytes=ascii'
+                b'&native=native&1=int&None=none'
+            ),
+            Headers({b'accept-encoding': [b'gzip']}),
+            None,
+        )
+
+    def test_request_query_param_seps(self):
+        """
+        When the characters ``&`` and ``#`` are passed to *params* as param
+        names or values they are percent-escaped in the URL.
+
+        This reproduces https://github.com/twisted/treq/issues/282
+        """
+        self.client.request('GET', 'http://example.com/', params=(
+            ('ampersand', '&'),
+            ('&', 'ampersand'),
+            ('octothorpe', '#'),
+            ('#', 'octothorpe'),
+        ))
+
+        self.agent.request.assert_called_once_with(
+            b'GET',
+            (
+                b'http://example.com/'
+                b'?ampersand=%26'
+                b'&%26=ampersand'
+                b'&octothorpe=%23'
+                b'&%23=octothorpe'
+            ),
+            Headers({b'accept-encoding': [b'gzip']}),
+            None,
+        )
+
     def test_request_merge_query_params(self):
         self.client.request('GET', 'http://example.com/?baz=bax',
                             params={'foo': ['bar', 'baz']})
@@ -139,6 +242,60 @@ class HTTPClientTests(TestCase):
 
         self.assertBody(b'hello')
 
+    def test_request_json_dict(self):
+        self.client.request('POST', 'http://example.com/', json={'foo': 'bar'})
+        self.agent.request.assert_called_once_with(
+            b'POST', b'http://example.com/',
+            Headers({b'Content-Type': [b'application/json; charset=UTF-8'],
+                     b'accept-encoding': [b'gzip']}),
+            self.FileBodyProducer.return_value)
+        self.assertBody(b'{"foo":"bar"}')
+
+    def test_request_json_tuple(self):
+        self.client.request('POST', 'http://example.com/', json=('foo', 1))
+        self.agent.request.assert_called_once_with(
+            b'POST', b'http://example.com/',
+            Headers({b'Content-Type': [b'application/json; charset=UTF-8'],
+                     b'accept-encoding': [b'gzip']}),
+            self.FileBodyProducer.return_value)
+        self.assertBody(b'["foo",1]')
+
+    def test_request_json_number(self):
+        self.client.request('POST', 'http://example.com/', json=1.)
+        self.agent.request.assert_called_once_with(
+            b'POST', b'http://example.com/',
+            Headers({b'Content-Type': [b'application/json; charset=UTF-8'],
+                     b'accept-encoding': [b'gzip']}),
+            self.FileBodyProducer.return_value)
+        self.assertBody(b'1.0')
+
+    def test_request_json_string(self):
+        self.client.request('POST', 'http://example.com/', json='hello')
+        self.agent.request.assert_called_once_with(
+            b'POST', b'http://example.com/',
+            Headers({b'Content-Type': [b'application/json; charset=UTF-8'],
+                     b'accept-encoding': [b'gzip']}),
+            self.FileBodyProducer.return_value)
+        self.assertBody(b'"hello"')
+
+    def test_request_json_bool(self):
+        self.client.request('POST', 'http://example.com/', json=True)
+        self.agent.request.assert_called_once_with(
+            b'POST', b'http://example.com/',
+            Headers({b'Content-Type': [b'application/json; charset=UTF-8'],
+                     b'accept-encoding': [b'gzip']}),
+            self.FileBodyProducer.return_value)
+        self.assertBody(b'true')
+
+    def test_request_json_none(self):
+        self.client.request('POST', 'http://example.com/', json=None)
+        self.agent.request.assert_called_once_with(
+            b'POST', b'http://example.com/',
+            Headers({b'Content-Type': [b'application/json; charset=UTF-8'],
+                     b'accept-encoding': [b'gzip']}),
+            self.FileBodyProducer.return_value)
+        self.assertBody(b'null')
+
     @mock.patch('treq.client.uuid.uuid4', mock.Mock(return_value="heyDavid"))
     def test_request_no_name_attachment(self):
 
